@@ -7,7 +7,7 @@ from typing import List, Any, Union
 
 from superbsapi import *
 
-from mx.exception import DBError
+from .exception import DBError, DataError
 from utils.conf.db_error import BS_NOERROR
 from utils.conf.constants import *
 from utils.conf.def_tree import *
@@ -46,7 +46,7 @@ type_map = {
 
 class TreeDB(object):
 
-    def __init__(self) -> None:
+    def __init__(self, *args, **kwargs) -> None:
         self.__chl = CBSHandleLoc()
 
     @staticmethod
@@ -76,7 +76,7 @@ class TreeDB(object):
         res = func(*args, **kwargs)
         return self.return_value(res)
 
-    def exec_bs(self, operate: str, *args: Any, **kwargs: Any) -> Union[int, tuple, None]:
+    def exec_bs(self, operate: str, *args: Any, **kwargs: Any) -> Union[int, str, tuple, None]:
         """
         用于执行bs开头的函数
 
@@ -87,6 +87,17 @@ class TreeDB(object):
         res = eval(operate)(self.__chl.GetConfHandle(), *args, **kwargs)
         return self.return_value(res)
 
+    def exec_class(self, operator: str, *args, **kwargs):
+        """
+        用于执行类方法
+
+        :param operator: 使用的函数
+        :return: 执行结果
+        """
+        func = getattr(CBSHandleLoc, operator)
+        res = func(*args, **kwargs)
+        return self.return_value(res)
+
     def begin(self) -> int:
         """
         开启事务
@@ -106,6 +117,26 @@ class TreeDB(object):
         """
         return self.exec_tree('Treedb_RollbackTransaction')
 
+    def create_index(self, name, name_type, flag=0):
+        """
+        创建索引
+        :param name: 要建立索引的字段名称
+        :param name_type: 字段类型
+        :param flag: 默认升序，设置BS_RITF_INDEX_DESC则为降序
+        :return:
+        """
+        return self.exec_bs('bs_treedb_create_index', [{'name': name, 'uFlag': flag,
+                                                        'uType': type_map.get(name_type) or name_type}])
+
+    def delete_index(self):
+        """
+        删除索引
+        :return:
+        """
+
+        # TODO 用到了再写吧
+        # return self.exec_bs('bs_treedb_delete_index')
+
     def open(self, main_key: str, sub_key: str = None, host: str = MxConfig.HOST, file: str = 'base',
              main_key_pwd: str = '', flag: int = None, port: int = MxConfig.PORT, path_flag: bool = False) -> "TreeDB":
         """
@@ -134,8 +165,7 @@ class TreeDB(object):
                 self.exec_tree('Treedb_ReopenMainKey', sub_key, flag, path_flag, main_key, main_key_pwd, file)
             except DBError:
                 try:
-                    logging.warning('您正使用Treedb_Alloc()重新连接数据库，如频繁看见此条警告，请确认是否正常')
-                    logging.warning('正在打开主键[%s]-子键[%s]' % (main_key, sub_key))
+                    logging.warning('正在重新连接数据库并打开主键[%s]-子键[%s]' % (main_key, sub_key))
                     self.__chl = CBSHandleLoc()
                     self.exec_tree('Treedb_Alloc', host, file, main_key, main_key_pwd, TRDB_OPKF_OPENEXIST, port)
                     self.exec_tree('Treedb_ReopenSubKey', sub_key, flag)
@@ -149,14 +179,27 @@ class TreeDB(object):
                     self.exec_tree('Treedb_ReopenMainKey', '', flag, path_flag, main_key, main_key_pwd, file)
                 except DBError:
                     try:
-                        logging.warning('您正使用Treedb_Alloc()重新连接数据库，如频繁看见此条警告，请确认是否正常')
-                        logging.warning('正在打开主键[%s]' % main_key)
+                        logging.warning('正在重新连接数据库并打开主键[%s]' % main_key)
                         self.__chl = CBSHandleLoc()
                         self.exec_tree('Treedb_Alloc', host, file, main_key, main_key_pwd, TRDB_OPKF_OPENEXIST, port)
                     except DBError as e:
                         raise DBError(e.err_code, '打开主键[%s]时' % main_key)
         return self
 
+    def main_keys(self, host: str = MxConfig.HOST, file: str = 'base', port: int = MxConfig.PORT) -> list:
+        """
+        获取所有主键
+        eg: sub_keys = db.open('MXSE', '1.SD', file='ccubase').main_keys()
+
+        :param host: 主机名
+        :param file: 数据库名
+        :param port: 端口号
+        :return: 主键列表
+        """
+        res_list = list()
+        self.exec_tree('Treedb_GetAllMainKeyNames', res_list, host, file, port)
+        return res_list
+
     def sub_keys(self) -> list:
         """
         获取当前键下所有的子键名
@@ -173,7 +216,7 @@ class TreeDB(object):
         获取当前键下，指定子键，指定属性的 key-value键值对
         eg: sub_items = db.open('MXSE', '1.SD', file='ccubase').sub_items(
                                 key_list=['1.SD.1', '1.SD.2'], prop_list='_target_ip')
-        当key_list为空的时候会获取全部，需要自行注意
+        <<<!!! 当key_list为空的时候会获取全部，需要自行注意 !!!>>>
             
         :param key_list: 要获取的子键列表
         :param prop_list: 要获取的属性列表
@@ -190,7 +233,7 @@ class TreeDB(object):
 
     def items(self, prop_list=None) -> dict:
         """
-        获取当前键下、指定属性的 key-value键值对
+        获取当前键下、指定属性的 key-value键值对, prop_list为空会获取所有属性
         eg: items = db.open('MXSE', '1.SD', file='ccubase').items(['max', 'mx_updatedatatime'])
         
         :param prop_list: 要获取属性
@@ -222,6 +265,19 @@ class TreeDB(object):
         """
         return self.exec_tree('Treedb_GetProperty', prop_name)
 
+    def insert_main_key(self, main_key: str, host: str, file: str,  main_pwd: str = '', port: int = 0):
+        """
+        插入主键
+        eg: db.insert_main_key('test', host='127.0.0.1', file='IOT')
+
+        :param main_key: 主键名
+        :param file: 文件名
+        :param host: 主机ip
+        :param main_pwd: 主键密码
+        :param port:
+        """
+        return self.exec_class('Treedb_CreateMainKey', host, file, main_key, main_pwd, port)
+
     def insert_sub_keys(self, sub_keys: Union[str, list], flag: int = TDDB_OPKF_CREATEDYNKEY) -> None:
         """
         批量插入子键
@@ -252,7 +308,7 @@ class TreeDB(object):
     @staticmethod
     def _check_item(item: tuple) -> int:
         """
-        校验item是否符合数据要求
+        校验item是否符合数据要求并返回value type
         """
         assert isinstance(item, tuple) and len(item) in [2, 3], '错误的数据类型，items列表中的数据应为元组且长度为2或3'
         assert type_map.get(item[2]) if len(item) == 3 else type_map.get(
@@ -260,7 +316,7 @@ class TreeDB(object):
         value_type = type_map[item[2]] if len(item) == 3 else type_map[type(item[1]).__name__]
         return value_type
 
-    def insert_items(self, items: List[tuple], overwrite=True) -> None:
+    def insert_items(self, items: List[tuple], overwrite=True) -> int:
         """
         批量插入数据
         eg: db.open('MXSE', '1.SD', file='ccubase').insert_items([('mxlabel', 'test'), ('mxid', 123)])
@@ -269,11 +325,13 @@ class TreeDB(object):
         :param overwrite: 是否覆盖原值
         """
         assert isinstance(items, list), '错误的数据类型，items应为列表'
+        data_list = list()
         for item in items:
-            value_type = self._check_item(item)
-            self.exec_bs('bs_treedb_insert_property', item[0], item[1], value_type, overwrite)
+            data_list.append({'name': item[0], 'value': item[1], 'valuetype': self._check_item(item)})
 
-    def insert_key_items(self, items: List[tuple], key: str = None, overwrite: bool = True) -> int:
+        return self.exec_bs('bs_treedb_insert_propertys', data_list, overwrite)
+
+    def insert_key_items(self, items: List[tuple], key: str = None) -> str:
         """
         批量插入子键及k,v
         eg: db.open('MXSE', '1.SD', file='ccubase').insert_key_items(
@@ -281,24 +339,26 @@ class TreeDB(object):
 
         :param items: 插入的数据, 如果元组传入三个元素，将第三个元素当作数据类型（见文首type_map），如果只传入两个，则自动获取数据类型
         :param key: 子键名称,不传自动生成
-        :param overwrite: 是否覆盖原值
+        :return: 生成的子键
         """
         if isinstance(items, tuple):
             items = [items]
         assert isinstance(items, list), '错误的数据类型，items应为列表'
         items_list = list()
         for item in items:
-            value_type = self._check_item(item)
-            items_list.append({'name': item[0], 'value': item[1], 'valuetype': value_type})
+            items_list.append({'name': item[0], 'value': item[1], 'valuetype': self._check_item(item)})
         return self.exec_bs('bs_treedb_insert_key_and_properties', key or '', TRDB_OPKF_OPENEXIST, items_list,
-                            overwrite)
+                            True)
 
     def delete(self, keys: Union[list, str, None] = None) -> None:
         """
         删除 单个子键/子键列表/自身
         eg: db.open('MXSE', '1.SD', file='ccubase').delete()
 
-        :param keys: 要删除的子键，不传会删除自身
+        :param keys: 要删除的子键。
+        <<<!!! 值为空时会删除自身 !!!>>>
+        <<<!!! 值为空时会删除自身 !!!>>>
+        <<<!!! 值为空时会删除自身 !!!>>>
         """
         if isinstance(keys, list):
             for key in keys:
@@ -388,19 +448,19 @@ class TreeDB(object):
         expression_list = list()
 
         for key, value in kwargs.items():
-            if re.match(r'^[_]?[[0-9A-Za-z]+[_]?[0-9A-Za-z]+]*?$', key):
+            try:
+                key, symbol = key.rsplit('__', 1)
+                if symbol not in symbol_map:
+                    raise DataError('查询操作错误！正确操作包含：gt、lt等，详情见TreeModel使用手册')
+            except ValueError:
                 symbol = 'e'
-            elif re.match(r'^[_]?[[0-9A-Za-z]+[_]?[0-9A-Za-z]+]*?__[0-9A-Za-z]+$', key):
-                key, symbol = key.split('__')
-                assert symbol in symbol_map, '查询操作错误！正确操作包含：gt、lt等，详情见TreeModel使用手册'
-            else:
-                raise ValueError('查询格式错误！正确示例：name="test", vLiData__gt=3，详情见TreeModel使用手册')
 
             temp = {'key': key, 'value_type': type(value).__name__, 'symbol': symbol, 'value': value}
 
             if symbol == 'in':
                 j = 0
                 assert isinstance(value, list), '查询格式错误！正确示例：a__in=[1, 3, 4, 5]，详情见TreeModel使用手册'
+                assert len(value) != 0, '使用in时列表不能为空'
                 for v in value:
                     temp = {'key': key, 'value_type': type(v).__name__, 'symbol': 'e', 'value': v}
                     args.append(temp)
